from django.http import HttpResponse, HttpResponseForbidden
from django.template import RequestContext, loader

from django.shortcuts import render
from django.utils.translation import ugettext as _

from uds.core.auths.auth import webLoginRequired
from uds.core.util.decorators import denyBrowsers

from uds.models.Provider import Provider
from uds.models.Authenticator import Authenticator
from uds.models.User import User
from uds.models.Group import Group
from uds.models.ServicesPool import ServicePool
from uds.models.Log import Log

import logging

logger = logging.getLogger(__name__)


@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def index(request):
    return render(request, 'termidesk/admin2/index.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def service_providers(request):
    return render(request, 'termidesk/admin2/providers.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def service_provider_info(request, provider_id):
    
    provider = Provider.objects.get(uuid=provider_id)
    context = {
        'provider': {
            'id':provider_id,
            'name':provider.name,
            'type': provider.data_type,
        }
        
    }
    return render(request, 'termidesk/admin2/provider.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def auth_domains(request):
    return render(request, 'termidesk/admin2/auth-domains.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def auth_domain_info(request, domain_id):
    
    domain = Authenticator.objects.get(uuid=domain_id)
    context = {
        'domain': {
            'id':domain_id,
            'name':domain.name,
            'type': domain.data_type,
        }
        
    }
    return render(request, 'termidesk/admin2/auth-domain.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def auth_domain_user_info(request, domain_id, user_id):
    
    domain = Authenticator.objects.get(uuid=domain_id)
    user = User.objects.get(uuid=user_id, manager=domain)
    context = {
        'domain': {
            'id':domain_id,
            'name':domain.name,
            'type': domain.data_type,
        },
        'domain_user': {
            'id': user_id,
            'name': user.name
        }
        
    }
    return render(request, 'termidesk/admin2/auth-domain-user.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def auth_domain_group_info(request, domain_id, group_id):
    domain = Authenticator.objects.get(uuid=domain_id)
    group = Group.objects.get(uuid=group_id, manager=domain)
    
    context = {
        'domain': {
            'id':domain_id,
            'name':domain.name,
            'type': domain.data_type,
        },
        'group': {
            'id': group_id,
            'name': group.name,
            'is_meta': group.is_meta,
            'meta_if_any': group.meta_if_any
        }
        
    }
    return render(request, 'termidesk/admin2/auth-domain-group.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def os_managers(request):
    return render(request, 'termidesk/admin2/os-managers.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def delivery_protocols(request):
    return render(request, 'termidesk/admin2/delivery-protocols.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def networks(request):
    return render(request, 'termidesk/admin2/components/networks.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def settings_gallery(request):
    return render(request, 'termidesk/admin2/settings/gallery.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def settings_service_pool_groups(request):
    return render(request, 'termidesk/admin2/settings/service-pool-groups.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def settings_sysconfig(request):
    return render(request, 'termidesk/admin2/settings/sysconfig.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def workplaces(request):
    return render(request, 'termidesk/admin2/workplaces.html')

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def workplace_info(request, workplace_id):
    
    workplace = ServicePool.objects.get(uuid=workplace_id)
    context = {
        'workplace': {
            'id':workplace_id,
            'name':workplace.name,
        }
        
    }
    
    return render(request, 'termidesk/admin2/workplace.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def workplace_as_info(request, workplace_id, service_id):
    
    workplace = ServicePool.objects.get(uuid=workplace_id)
    service = workplace.assignedUserServices().get(uuid=service_id)
    context = {
        'workplace': {
            'id':workplace_id,
            'name':workplace.name,
        },
        'service': {
            'id': service_id,
            'name': service.friendly_name,
        }
        
    }
    
    return render(request, 'termidesk/admin2/workplace_assigned_service.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def workplace_cs_info(request, workplace_id, service_id):
    
    workplace = ServicePool.objects.get(uuid=workplace_id)
    service = workplace.cachedUserServices().get(uuid=service_id)
    context = {
        'workplace': {
            'id':workplace_id,
            'name':workplace.name,
        },
        'service': {
            'id': service_id,
            'name': service.friendly_name,
        }
        
    }
    
    return render(request, 'termidesk/admin2/workplace_cached_service.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def monitoring(request):
    
    #logs = Log.objects.all().order_by('-created')[0:1000]
    context = {
       # 'logs': logs
    }
    return render(request, 'termidesk/admin2/monitoring.html', context)

@denyBrowsers(browsers=['ie<9'])
@webLoginRequired(admin=True)
def tmpl(request, template):
    try:
        t = loader.get_template('termidesk/admin2/tmpl/' + template + ".html")
        c = RequestContext(request)
        resp = t.render(c)
    except Exception as e:
        logger.debug('Exception getting template: {0}'.format(e))
        resp = _('requested a template that do not exist')
    return HttpResponse(resp, content_type="text/plain")