# -*- coding: utf-8 -*-
'''
Created on 16 янв. 2019 г.

@author: dvd
'''
from __future__ import unicode_literals

import os
import base64
import datetime
import json

from Crypto.Signature import PKCS1_PSS
from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
import binascii

import logging

logger = logging.getLogger(__name__)

def generate_salt():
    salt = os.urandom(16)
    return str(base64.b64encode(salt))

class TokenEncoder(object):
    
    @staticmethod
    def _formatDate(d):
        return d.strftime("%Y%m%d%H%M%S")

    def __init__(self, sign_key, lifetime=5):
        self._sign_key = sign_key
        self._lifetime = lifetime
    
    def encode(self, data):
        d = {
            'salt': generate_salt(),
            'digest': 'sha256',
            'validFrom': self._formatDate(datetime.datetime.utcnow()),
            'validTo': self._formatDate(
                datetime.datetime.utcnow() + datetime.timedelta(
                    seconds=self._lifetime
                )
            ),
            'data': data
        }
        digest_str = json.dumps(d, sort_keys=True)
        #logger.info("digest_str: {}".format(digest_str))
        key = RSA.importKey(self._sign_key)
        h = SHA256.new(digest_str)
    
        signer = PKCS1_PSS.new(key)
        signature = signer.sign(h)
        
        d['signature'] = binascii.hexlify(signature)
        #logger.info("signature: {}".format(d['signature']))
        return base64.b64encode(json.dumps(d))

class TokenDecoder(object):
    
    def __init__(self, verification_key, lifetime=5):
        self._verification_key = verification_key
        self._lifetime = lifetime
    
    @staticmethod
    def _parseDate(d):
        return datetime.datetime.strptime(d, '%Y%m%d%H%M%S')
    
    def decode(self, ticket):
        decoded = json.loads(base64.b64decode(ticket))
        #logger.info("signature: {}".format(decoded['signature']))
        signature = binascii.unhexlify(decoded['signature'])
        del(decoded['signature'])
        
        key = RSA.importKey(self._verification_key)
        digest_str = json.dumps(decoded,sort_keys=True)
        #logger.info("digest_str: {}".format(digest_str))
        h = SHA256.new(digest_str)
        
        verifier = PKCS1_PSS.new(key)
        if not verifier.verify(h, signature):
            raise ValueError('Invalid ticket signature')
        
        if not (
            self._parseDate(decoded['validFrom']) <=
            datetime.datetime.utcnow() <=
            self._parseDate(decoded['validTo'])
        ):
            raise ValueError('Ticket lifetime expired')
    
        return decoded['data']