# -*- coding: utf-8 -*-
'''
Created on 22 авг. 2018 г.

@author: dvd
'''
from __future__ import unicode_literals

from uds.core import Module
from uds.core.ui.UserInterface import gui
from uds.REST import ResponseError
from uds.core.Environment import Environment
from uds.core.auths import Authenticator
from uds.models import Authenticator as AuthenticatorModel
from uds.models import Group
from uds.models import Transport
import logging

logger = logging.getLogger(__name__)

class UsersUIHelper(object):
      
    @staticmethod
    def getResources(parameters):
        logger.debug('Parameters received by getResources Helper: {0}'.format(parameters))
        
        data = [
            {
                'name': 'groups',
                'values': [{'id': 'g1', 'text': 'group1'}, {'id': 'g2', 'text': 'group2'}]
            }
        ]

        logger.debug('return data: {0}'.format(data))
        return data

class UsersUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(UsersUI, self).__init__(environment, values)
        self._parent = parent
    
    name = gui.TextField(order=1,
        length=64,
        label='Логин',
        tooltip='Содержит логин пользователя',
        required=True,
        search = False,
    )
    
    real_name = gui.TextField(order=2,
        length=64,
        label='Имя',
        tooltip='Содержит имя пользователя',
        required=True,
    )
    
    comments = gui.TextField(
        order = 3,
        label = 'Комментарий',
        length= 256,
    )
    
    state = gui.ChoiceField(
        order = 4,
        label = 'Статус',
        values=[
            gui.choiceItem('A', 'Активный'),
            gui.choiceItem('I', 'Неактивный'),
            gui.choiceItem('B', 'Заблокирован'),
        ],
        defvalue='A',
        required=True
        
    )
    
    staff_member = gui.CheckBoxField(
        order = 5,
        label = 'Персонал',
        defvalue=gui.FALSE
    )
    
    is_admin = gui.CheckBoxField(
        order = 6,
        label = 'Администратор',
        defvalue=gui.FALSE
    )
    
#     password = gui.PasswordField(
#         order=7,
#         length=32, 
#         label='Пароль',  
#         required=False
#     )
    
    groups = gui.MultiChoiceField(
        label='Группы',
        order=8,
#         fills={
#             'callbackName': 'userFillGroups',
#             'function': UsersUIHelper.getResources,
#             'parameters': ['id']
#         },
        required=False
    )
    
    
    # Override getGuiDescription method to handle password field
    # Т.к. некоторым аутентификаторам оно требуется, а некоторым нет
    @classmethod
    def guiDescription(cls, obj=None):
        #logger.debug("Inside overriden getGuiDesc")
        result = super(UsersUI, cls).guiDescription(obj)
        
        if obj is not None:
            pi = obj._parent.getInstance()
            need_pwd = pi.needsPassword
            #logger.debug("Need pwd: {}".format(need_pwd))
            
            if need_pwd:
                password = gui.PasswordField(
                    order=7,
                    length=32, 
                    label='Пароль',  
                    required=False
                )
                
                result.append({'name': 'password', 'gui': password.guiDescription(), 'value': ''})
            
        return result
        
    def initGui(self):
        '''
        Loads required values inside
        '''
        pi = self._parent.getInstance()
        
        try:
            self.name.label = pi.userNameLabel
        except AttributeError:
            pass
        
        groups = self._parent.groups.all().order_by('name')
        vals = []
        for g in groups:
            vals.append(gui.choiceItem(g.uuid, g.name))

        self.groups.setValues(vals)
        
        # Если метод отличается от родительского, 
        # то поиск пользователей возможен
        #logger.debug('SearchUsers: {} neq {}  = {}'.format(pi.__class__.searchUsers, Authenticator.searchUsers,(pi.__class__.searchUsers != Authenticator.searchUsers)))
        if pi.__class__.searchUsers != Authenticator.searchUsers:
            self.name.search = True
        
class GroupUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(GroupUI, self).__init__(environment, values)
        self._parent = parent
    
    name = gui.TextField(order=1,
        length=64,
        label='Имя группы',
        required=True,
        rdonly=True,
        search = False,
    )
    
    comments = gui.TextField(
        order = 3,
        label = 'Комментарий',
        length= 256,
    )
    
    state = gui.ChoiceField(
        order = 4,
        label = 'Статус',
        values=[
            gui.choiceItem('A', 'Активный'),
            gui.choiceItem('I', 'Неактивный'),
        ],
        defvalue='A',
        required=True
        
    )
    
    type = gui.HiddenField(order=10,
        length=64,
        required=True,
        defvalue='group'
    )
    
    def initGui(self):
        pi = self._parent.getInstance()
        
        try:
            self.name.label = pi.groupNameLabel
        except AttributeError:
            pass
        
        # Если метод отличается от родительского, 
        # то поиск групп возможен
        if pi.__class__.searchGroups != Authenticator.searchGroups:
            self.name.search = True

class MetaGroupUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(MetaGroupUI, self).__init__(environment, values)
        self._parent = parent
    
    name = gui.TextField(order=1,
        length=64,
        label='Имя метагруппы',
        required=True,
        search = False,
    )
    
    meta_if_any = gui.CheckBoxField(
        order = 2,
        label = 'Тип совпадения',
        on_label = 'Любой',
        off_label = 'Все',
    )
    
    comments = gui.TextField(
        order = 3,
        label = 'Комментарий',
        length= 256,
    )
    
    state = gui.ChoiceField(
        order = 4,
        label = 'Статус',
        values=[
            gui.choiceItem('A', 'Активный'),
            gui.choiceItem('I', 'Неактивный'),
        ],
        defvalue='A',
        required=True
        
    )
    
    type = gui.HiddenField(order=10,
        length=64,
        required=True,
        defvalue='meta'
    )
    
    groups = gui.MultiChoiceField(
        label='Группы',
        order=8,
        required=False
    )
    
    def initGui(self):
        '''
        Loads required values inside
        '''
        
        pi = self._parent.getInstance()
        
        try:
            self.name.label = pi.groupNameLabel
        except AttributeError:
            pass
        
        groups = self._parent.groups.filter(is_meta=False).order_by('name')
        vals = []
        for g in groups:
            vals.append(gui.choiceItem(g.uuid, g.name))

        self.groups.setValues(vals)

class ServicePoolsGroupUIHelper(object):
      
    @staticmethod
    def getResources(parameters):
        domain = AuthenticatorModel.objects.get(uuid=parameters['domain'])
        groups = Group.objects.filter(manager=domain)
        values = []
        for g in groups:
            values.append({
                'id': g.uuid,
                'text': g.name,
            })
                           
        data = [
            {
                'name': 'id',
                'values': values,
            }
        ]

        return data

class ServicePoolsGroupUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(ServicePoolsGroupUI, self).__init__(environment, values)
        self._parent = parent
    
    domain = gui.ChoiceField(
        label = 'Домен аутентификации',
        required=True,
        fills={
            'callbackName': 'domainFillGroups',
            'function': ServicePoolsGroupUIHelper.getResources,
            'parameters': ['domain']
        },
    )
    
    id = gui.ChoiceField(
        label = 'Группа',
        required=True
    )
    
    def initGui(self):
        
        domains = AuthenticatorModel.objects.all()
        vals = []
        for domain in domains:
            vals.append(gui.choiceItem(domain.uuid, domain.name))

        self.domain.setValues(vals)

class ServicePoolsTransportUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(ServicePoolsTransportUI, self).__init__(environment, values)
        self._parent = parent
    
    id = gui.ChoiceField(
        label = 'Протокол доставки',
        required=True
    )
    
    def initGui(self):
        
        transports = Transport.objects.all()
        vals = []
        for transport in transports:
            vals.append(gui.choiceItem(transport.uuid, transport.name))

        self.id.setValues(vals)    

class ServicePoolsPublicationUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(ServicePoolsPublicationUI, self).__init__(environment, values)
        self._parent = parent
    
    changelog = gui.InputField(
        label = 'Причина',
        required=False
    )
    
    def initGui(self):
        
        pass 
