# -*- coding: utf-8 -*-
'''
Created on 6 марта 2017 г.

@author: dvd
'''

from __future__ import unicode_literals

from django.utils.translation import ugettext as _

from uds.REST import Handler
from uds.models import TicketStore

from uds.core.managers import userServiceManager, cryptoManager
from uds.core.services.Exceptions import ServiceNotReadyError, MaxServicesReachedError, ServiceAccessDeniedByCalendar

import datetime
import six

import logging

logger = logging.getLogger(__name__)

class Enable(Handler):
    path = 'termidesk'
    needs_admin = False

    @staticmethod
    def result(result='', error=None):
        '''
        Returns a result for a Ticket request
        '''
        res = {'result': result, 'date': datetime.datetime.now()}
        if error is not None:
            res['error'] = error
        return res

    def get(self):
        '''
        Processes get requests, currently none
        '''
        logger.debug("Ticket args for GET: {0}".format(self._args))
        
        idService, idTransport = self._args

        logger.debug('idService: {}, idTransport: {}'.format(idService, idTransport))
        url = ''
        error = _('Service not ready. Please, try again in a while.')
        
        try:
            res = userServiceManager().getService(self._user, self._request.ip, idService, idTransport, doTest=False)
            scrambler = cryptoManager().randomString(32)
            
            # В данной реализации не поддерживается получение пароля 
            # пользователя из веб сессии. Возможно, в будущем будет реализован 
            # другой способ получения пароля.
            # Траснпорты, где требуются логин и пароль пользователя, могут 
            # работать некорректно.
            #password = cryptoManager().xor(webPassword(self._request), scrambler)
            password = cryptoManager().xor('', scrambler)
    
            _x, userService, _x, trans, _x = res
    
            data = {
                'service': 'A' + userService.uuid,
                'transport': trans.uuid,
                'user': self._user.uuid,
                'password': password
            }
    
            ticket = TicketStore.create(data)
            error = ''
            data = {
                'ticket': ticket,
                'scrambler': scrambler
            }
            
        except ServiceNotReadyError as e:
            logger.debug('Service not ready')
            error += 'Service not ready (code {0:04X})'.format(e.code)
            return Enable.result(error=error)
        except MaxServicesReachedError:
            error = 'Number of service reached MAX for service pool "{}"'.format(idService)
            logger.info(error)
            return Enable.result(error=error)
        except ServiceAccessDeniedByCalendar:
            error = 'Access tried to a calendar limited access pool "{}"'.format(idService)
            logger.info(error)
            return Enable.result(error=error)
        except Exception as e:
            logger.exception('Error')
            error = six.text_type(e)
            return Enable.result(error=six.text_type(e))
    
    
        return Enable.result(data, error)
    