# -*- coding: utf-8 -*-
'''
Created on 4 окт. 2018 г.

@author: dvd
'''
from __future__ import unicode_literals

from uds.core.util.Config import Config as ConfigParams
from uds.core.util.Config import GlobalConfig
from uds.core.util.Config import CLUSTER_SECTION
from uds.core.util.Config import SECURITY_SECTION
from uds.core.util.Config import GLOBAL_SECTION
from uds.REST import Handler
from uds.core import auths
from uds.core import transports
import logging

logger = logging.getLogger(__name__)

TRANSLATED_SECTIONS = {
    'UDS': 'Общие',
    'Security': 'Безопасность',
    'IPAUTH': 'Аутентификация',
}

REVERSED_SECTIONS = { v:k for k,v in TRANSLATED_SECTIONS.iteritems() }

TRANSLATED_KEYS = {
    'UDS Theme': 'Theme',
    'UDS Theme Enhaced': 'Theme Enhanced',
}

BASE_SUPRESSED_KEYS = {
    CLUSTER_SECTION: [
        # Параметры кластерных поставщиков ресурсов (ClusteredServiceProvider)
        # Не используются
        GlobalConfig.CLUSTER_MIGRATE_CPULOAD.key(),
        GlobalConfig.CLUSTER_ELEGIBLE_CPULOAD.key(),
        GlobalConfig.CLUSTER_MIGRATE_MEMORYLOAD.key(),
        GlobalConfig.CLUSTER_ELEGIBLE_MEMORYLOAD.key()
    ],
    SECURITY_SECTION: [
        # Теперь всегда за nginx прокси
        GlobalConfig.BEHIND_PROXY.key(),
    ],
    GLOBAL_SECTION: [
        #  Календарь не используется
        GlobalConfig.LIMITED_BY_CALENDAR_TEXT.key(),
        # Параметр времени перезагрузки страницы не используется
        # Возможно будет удален в будущем
        GlobalConfig.RELOAD_TIME.key(),
        # В новой админке не используется
        # Возможно будет удален в будущем
        GlobalConfig.UDS_THEME_VISUAL.key(),
        # Пользовательские настройки не используются
        # Возможно параметр будет удален в будущем
        GlobalConfig.PREFERENCES_ALLOWED.key(),
        # Дополнительный текст при входе не используется
        # Возможно параметр будет удален в будущем
        GlobalConfig.CUSTOM_HTML_LOGIN.key(),
    ]
}

FIELD_INFO = {
    SECURITY_SECTION: {
        'Master Key': {
            'label': 'Мастер ключ',
            'tooltip': '',
        },
        GlobalConfig.TRUSTED_SOURCES.key(): {
            'label': 'Доверенные хосты',
            'tooltip': '',
        },
        GlobalConfig.ADMIN_IDLE_TIME.key(): {
            'label': 'Длительность сессии администратора',
            'tooltip': '',
        },
        GlobalConfig.SUPER_USER_LOGIN.key(): {
            'label': 'Логин администратора',
            'tooltip': '',
        },
        GlobalConfig.SUPER_USER_PASS.key(): {
            'label': 'Пароль администратора',
            'tooltip': '',
        },
        GlobalConfig.SUPER_USER_ALLOW_WEBACCESS.key(): {
            'label': 'Доступ администратора к веб-части',
            'tooltip': '',
        },
        GlobalConfig.SUPER_USER_PAM.key(): {
            'label': 'Системная аутентификация администратора',
            'tooltip': '',
        },
        GlobalConfig.USER_SESSION_LENGTH.key(): {
            'label': 'Длительность сессии пользователя',
            'tooltip': '',
        },
        GlobalConfig.EXCLUSIVE_LOGOUT.key(): {
            'label': 'Эксклюзивный выход',
            'tooltip': '',
        },
        GlobalConfig.HONOR_CLIENT_IP_NOTIFY.key(): {
            'label': 'Использовать анонсируемый IP клиента',
            'tooltip': '',
        },
        
    },
    GLOBAL_SECTION: {
        # Пример
         GlobalConfig.UDS_THEME.key(): {
             'label':'Тема оформления',
             'tooltip': 'Тема оформления',
        },
        GlobalConfig.AUTORUN_SERVICE.key(): {
            'label': 'Автозапуск рабочего места',
            'tooltip': '',
        },
        GlobalConfig.LOGIN_URL.key(): {
            'label': 'URL входа',
            'tooltip': '',
        },
        GlobalConfig.SCHEDULER_THREADS.key(): {
            'label': 'Количество потоков планировщика задач',
            'tooltip': '',
        },
        GlobalConfig.CLEANUP_CHECK.key(): {
            'label': 'Интервал очистки информационных объектов',
            'tooltip': '',
        },
        GlobalConfig.KEEP_INFO_TIME.key(): {
            'label': 'Время хранения информационных объектов',
            'tooltip': '',
        },
        GlobalConfig.MAX_PREPARING_SERVICES.key(): {
            'label': 'Максимум подготавливаемых рабочих мест',
            'tooltip': '',
        },
        GlobalConfig.MAX_REMOVING_SERVICES.key(): {
            'label': 'Максимум удаляемых рабочих мест',
            'tooltip': '',
        },
        GlobalConfig.MAX_INITIALIZING_TIME.key(): {
            'label': 'Максимальное время инициализации рабочего места',
            'tooltip': '',
        },
        GlobalConfig.IGNORE_LIMITS.key(): {
            'label': 'Не учитывать максимальные ограничения',
            'tooltip': '',
        },
        GlobalConfig.MAX_LOGIN_TRIES.key(): {
            'label': 'Макимум попыток входа',
            'tooltip': '',
        },
        GlobalConfig.LOGIN_BLOCK.key(): {
            'label': 'Время блокировки входа',
            'tooltip': '',
        },
        GlobalConfig.REDIRECT_TO_HTTPS.key(): {
            'label': 'Перенаправлять на HTTPS',
            'tooltip': '',
        },
        GlobalConfig.REMOVAL_CHECK.key(): {
            'label': 'Интервал проверки для удаления объектов',
            'tooltip': '',
        },
        GlobalConfig.USER_SERVICE_CLEAN_NUMBER.key(): {
            'label': 'Количество удаляемых рабочих мест за один проход',
            'tooltip': '',
        },
        GlobalConfig.CACHE_CHECK_DELAY.key(): {
            'label': 'Интервал проверок кеша рабочих мест',
            'tooltip': '',
        },
        GlobalConfig.CHECK_UNUSED_TIME.key(): {
            'label': 'Интервал проверок неиспользуемых рабочих мест',
            'tooltip': '',
        },
        GlobalConfig.DELAYED_TASKS_THREADS.key(): {
            'label': 'Количество потоков фоновых задач',
            'tooltip': '',
        },
        GlobalConfig.DISALLOW_GLOBAL_LOGIN.key(): {
            'label': 'Запретить глобальный вход',
            'tooltip': '',
        },
        GlobalConfig.MAX_LOGS_PER_ELEMENT.key(): {
            'label': 'Максимум записей в журнале для объектов',
            'tooltip': '',
        },
        GlobalConfig.RESTRAINT_COUNT.key(): {
            'label': 'Количество ошибок для ограничения фонда',
            'tooltip': '',
        },
        GlobalConfig.RESTRAINT_TIME.key(): {
            'label': 'Интервал отслеживания ошибок в фонде',
            'tooltip': '',
        },
        GlobalConfig.SESSION_EXPIRE_TIME.key(): {
            'label': 'Срок действия устаревшей публикации',
            'tooltip': '',
        },
        GlobalConfig.STATS_DURATION.key(): {
            'label': 'Срок хранения статистики',
            'tooltip': '',
        },
        
    },
    'IPAUTH': {
        'autoLogin': {
            'label': 'IP аутентификация',
        }
    }
}

REVERSED_KEYS = { v:k for k,v in TRANSLATED_KEYS.iteritems() }

class TConfig(Handler):
    needs_admin = True  # By default, staff is lower level needed

    def get(self):
        res = {}
        addCrypt = self.is_admin()
        
        SUPRESSED_KEYS = dict(BASE_SUPRESSED_KEYS)
        
        # Скрываем поля неустановленных плагинов
        nx_trans_plugin = transports.factory().lookup('NXTransport')
        if nx_trans_plugin is None:
            SUPRESSED_KEYS['NX'] = ['downloadUrl', 'downloadUrlMACOS']
            
        ip_auth_plugin = auths.factory().lookup('IPAuth')
        if ip_auth_plugin is None:
            SUPRESSED_KEYS['IPAUTH'] = ['autoLogin']
        
        for param in ConfigParams.enumerate():
            if param.isCrypted() is True and addCrypt is False:
                continue
            
            section = param.section()
            key = param.key()
            label = FIELD_INFO.get(section,{}).get(key,{}).get('label', key)
            tooltip = FIELD_INFO.get(section,{}).get(key,{}).get('tooltip','')
            
            if section in SUPRESSED_KEYS:
                section_keys = SUPRESSED_KEYS[section]
                if key in section_keys:
                    # Пропускаем ключи которые не надо показывать
                    continue
            
            if TRANSLATED_SECTIONS.has_key(section):
                section = TRANSLATED_SECTIONS[section]
                
            # add section if it do not exists
            if section not in res:
                res[section] = {}
                
            if TRANSLATED_KEYS.has_key(key):
                key = TRANSLATED_KEYS[key]
                
            res[section][key] = {
                'label': label,
                'value': param.get(),
                'crypt': param.isCrypted(),
                'longText': param.isLongText(),
                'type': param.getType(),
                'params': param.getParams(),
                'tooltip': tooltip,
            }
        logger.debug('Configuration: {0}'.format(res))
        return res

    def put(self):
        for section, secDict in self._params.iteritems():
            if REVERSED_SECTIONS.has_key(section):
                section = REVERSED_SECTIONS[section]
                
            for key, vals in secDict.iteritems():
                if REVERSED_KEYS.has_key(key):
                    key = REVERSED_KEYS[key]
                ConfigParams.update(section, key, vals['value'])
        return 'ok'
