# -*- coding: utf-8 -*-
'''
Created on Nov 8, 2019

@author: dvd
'''
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from uds.core import services
from uds.core.util.State import State
import logging

logger = logging.getLogger(__name__)
import json

class OVirtStaticDeployment(services.UserDeployment):
    suggestedTime = 5

    def initialize(self):
        self._ip = self.service().getHostname()
        self._vmid = self.service().vmId.value
        self._state = State.FINISHED
        self._reason = ''
    
    # Serializable needed methods
    def marshal(self):
        data = {
            'version': 1,
            'ip': self._ip,
            'vm_id': self._vmid,
            'state': self._state,
            'reason': self._reason,
        }
        
        return json.dumps(data)
        

    def unmarshal(self, str_):
        data = json.loads(str_)
        if data['version']==1:
            self._ip = data.get('ip','')
            self._vmid = data['vm_id']
            self._reason = data['reason']
            self._state = data['state']
        else:
            raise Exception("Unsupported serialized data version {}".format(data['version']))
    
    def setIp(self, ip):
        logger.debug('Setting IP to %s (ignored)' % ip)

    def getIp(self):
        return self._ip

    def getName(self):
        return self._vmid

    def getUniqueId(self):
        return self._vmid

    def setReady(self):
        self._state = State.FINISHED
        return self._state

    def deployForUser(self, user):
        logger.debug("Starting deploy of {0} for user {1}".format(self._ip, user))
        self.dbservice().setInUse(True)
        self.dbservice().save()
        self._state = State.FINISHED
        return self._state

    def checkState(self):
        return self._state

    def finish(self):
        pass

    def reasonOfError(self):
        '''
        If a publication produces an error, here we must notify the reason why it happened. This will be called just after
        publish or checkPublishingState if they return State.ERROR
        '''
        return self._reason

    def cancel(self):
        return self.destroy()

    def destroy(self):
        self._state = State.FINISHED
        return self._state
    
    def getConsoleConnection(self):
        return self.service().getConsoleConnection(self._vmid)