# -*- coding: utf-8 -*-

'''
@author: Dmitry Dvoeglazov
'''
from pkg_resources import iter_entry_points

def __init__():
    '''
    Импортирует дочерние модули и затем регистрирует классы в реестре
    '''
    import os.path
    import pkgutil
    import sys
    import logging
    from importlib import import_module
    from uds.core import services

    logger = logging.getLogger(__name__)
    
    logger.debug("Searching for provider plugins...")
    for entry_point in iter_entry_points('termidesk.providers'):
        logger.debug("Found provider plugin: {}".format(entry_point.name))
        plugin_class = entry_point.load()
        logger.debug("Loaded plugin: {}".format(plugin_class))
        logger.debug("Plugin location: {}".format(sys.modules[plugin_class.__module__].__file__))
        services.factory().insert(plugin_class)
        logger.debug('Registered {} class with service factory'.format(str(plugin_class)))
    
    # Динамически импортируем реализации реализации сервис-провайдеров
    # из дочерних пакетов
    # При импорте пакеты должны регистрировать сервис провайдеров
    # в общем реестре (ServiceProviderFactory)
    pkgpath = os.path.dirname(sys.modules[__name__].__file__)
    for _, name, _ in pkgutil.iter_modules([pkgpath]):
        import_module('.{}'.format(name), __package__)
        logger.debug("Imported {} module from {}".format(name, __package__))

    for provider in [services.ServiceProvider, services.ClusteredServiceProvider]:

        for cls in provider.__subclasses__(): # pylint: disable=maybe-no-member
            if not str(cls.__module__).startswith('{}.'.format(__package__)):
                continue
            services.factory().insert(cls)
            logger.debug('Registered {} class with service factory'.format(str(cls)))

__init__()
del __init__
