// Generated by CoffeeScript 2.3.0
(function() {
  // api.js
  // jshint strict: true
  "use strict";
  var $, BasicModelRest, Cache, DetailModelRestApi, api, ref;

  this.api = (ref = this.api) != null ? ref : {};

  $ = jQuery;

  api = this.api;

  api.debug = false;

  api.permissions = {
    NONE: 0,
    READ: 32,
    MANAGEMENT: 64,
    ALL: 96
  };

  api.doLog = function(...args) {
    if (api.debug) {
      try {
        console.log.apply(window, args);
      } catch (error) {}
    }
  };

  api.cacheTable = {};

  api.cache = function(cacheName) {
    return new Cache(cacheName);
  };

  api.cache.clear = function(cacheName) {
    if (cacheName == null) {
      api.cacheTable = {};
    } else {
      api.cacheTable[cacheName] = {};
    }
  };

  api.url_for = function(path, type = 'rest') {
    api.doLog('Url for: ', path, ', ', type);
    switch (type) {
      case "template":
        return api.config.template_url + path;
      case "rest":
        return api.config.base_url + path;
      default:
        api.doLog('Type of url not found: ' + type);
        throw "Type of url not found: " + type;
    }
  };

  api.defaultFail = function(jqXHR, textStatus, errorThrown) {
    api.doLog(jqXHR, ", ", textStatus, ", ", errorThrown);
  };

  api.getJson = function(path, options) {
    var fail_fnc, success_fnc, url;
    options = options || {};
    success_fnc = options.success || function() {};
    fail_fnc = options.fail || api.defaultFail;
    url = api.url_for(path);
    api.doLog("Ajax GET Json for \"" + url + "\"");
    $.ajax({
      url: url,
      type: options.method || "GET",
      dataType: "json",
      success: function(data) {
        api.doLog("Success on GET \"" + url + "\".");
        //api.doLog "Received ", data
        success_fnc(data);
      },
      error: function(jqXHR, textStatus, errorThrown) {
        api.doLog("Error on GET \"" + url + "\". ", textStatus, ", ", errorThrown);
        fail_fnc(jqXHR, textStatus, errorThrown);
      },
      beforeSend: function(request) {
        request.setRequestHeader(api.config.auth_header, api.config.token);
      }
    });
  };

  api.putJson = function(path, data, options) {
    var fail_fnc, success_fnc, url;
    options = options || {};
    success_fnc = options.success || function() {};
    fail_fnc = options.fail || api.defaultFail;
    url = api.url_for(path);
    api.doLog("Ajax PUT Json for \"" + url + "\"");
    $.ajax({
      url: url,
      type: options.method || "PUT",
      dataType: "json",
      data: JSON.stringify(data),
      success: function(data) {
        api.doLog("Success on PUT \"" + url + "\".");
        // api.doLog "Received ", data
        success_fnc(data);
      },
      error: function(jqXHR, textStatus, errorThrown) {
        api.doLog("Error on PUT \"" + url + "\". ", textStatus, ", ", errorThrown);
        fail_fnc(jqXHR, textStatus, errorThrown);
      },
      beforeSend: function(request) {
        request.setRequestHeader(api.config.auth_header, api.config.token);
      }
    });
  };

  api.deleteJson = function(path, options) {
    var fail_fnc, success_fnc, url;
    options = options || {};
    success_fnc = options.success || function() {};
    fail_fnc = options.fail || api.defaultFail;
    url = api.url_for(path);
    api.doLog("Ajax DELETE Json for \"" + url + "\"");
    $.ajax({
      url: url,
      type: "DELETE",
      dataType: "json",
      success: function(data) {
        api.doLog("Success on DELETE \"" + url + "\".");
        // api.doLog "Received ", data
        success_fnc(data);
      },
      error: function(jqXHR, textStatus, errorThrown) {
        api.doLog("Error on DELETE \"" + url + "\". ", textStatus, ", ", errorThrown);
        fail_fnc(jqXHR, textStatus, errorThrown);
      },
      beforeSend: function(request) {
        request.setRequestHeader(api.config.auth_header, api.config.token);
      }
    });
  };

  Cache = class Cache {
    constructor(cacheName) {
      api.cacheTable[cacheName] = api.cacheTable[cacheName] || {};
      this.name = cacheName;
      this.cache = api.cacheTable[cacheName];
    }

    get(key, not_found_fnc) {
      not_found_fnc = not_found_fnc || function() {
        return null;
      };
      if (this.cache[key] == null) {
        this.cache[key] = not_found_fnc();
      }
      return this.cache[key] || void 0;
    }

    put(key, value) {
      this.cache[key] = value;
    }

  };

  BasicModelRest = class BasicModelRest {
    constructor(path, options) {
      options = options || {};
      path = path || "";
      // Requests paths
      this.path = path;
      this.getPath = options.getPath || path;
      this.logPath = options.logPath || path;
      this.putPath = options.putPath || path;
      this.testPath = options.testPath || (path + "/test");
      this.delPath = options.delPath || path;
      this.typesPath = options.typesPath || (path + "/types");
      this.guiPath = options.guiPath || (path + "/gui");
      this.tableInfoPath = options.tableInfoPath || (path + "/tableinfo");
      this.cache = api.cache("bmr" + path);
    }

    _requestPath(path, options) {
      var $this, cacheKey, fail_fnc, success_fnc;
      api.doLog("Requesting ", path, options);
      options = options || {};
      success_fnc = options.success || function() {
        api.doLog("success function not provided for " + path);
      };
      fail_fnc = options.fail || function(jqXHR, textStatus, errorThrown) {
        api.doLog("failFnc not provided for " + path);
        gui.tools.unblockUI();
        return gui.notify('Error ocurred: ' + textStatus, 'danger');
      };
      cacheKey = options.cacheKey || path;
      api.doLog('CacheKey ', cacheKey);
      if (path === ".") {
        success_fnc({});
        return;
      }
      if (cacheKey !== "." && this.cache.get(cacheKey)) {
        api.doLog("Cache SUCCESS for " + cacheKey);
        success_fnc(this.cache.get(cacheKey));
      } else {
        api.doLog("Cache FAIL for " + cacheKey);
        $this = this;
        api.doLog('Obtaining json for ', path);
        api.getJson(path, {
          success: function(data) {
            if (cacheKey !== ".") {
              $this.cache.put(cacheKey, data);
            }
            success_fnc(data);
          },
          fail: fail_fnc
        });
      }
    }

    get(options) {
      var path;
      options = options || {};
      path = this.getPath;
      if (options.id) {
        path += "/" + options.id;
      }
      api.doLog("get Options: ", options, path);
      return this._requestPath(path, {
        cacheKey: ".",
        success: options.success,
        fail: options.fail
      });
    }

    list(success_fnc, fail_fnc) {
      return this.get({
        id: "",
        success: success_fnc,
        fail: fail_fnc
      });
    }

    overview(success_fnc, fail_fnc) {
      return this.get({
        id: "overview",
        success: success_fnc,
        fail: fail_fnc
      });
    }

    item(itemId, success_fnc, fail_fnc) {
      return this.get({
        id: itemId,
        success: success_fnc,
        fail: fail_fnc
      });
    }

    getLogs(itemId, success_fnc, fail_fnc) {
      var path;
      path = this.logPath + "/" + itemId + "/" + "log";
      return this._requestPath(path, {
        cacheKey: ".",
        success: success_fnc,
        fail: fail_fnc
      });
    }

    put(data, options) {
      var path;
      options = options || {};
      path = this.putPath;
      if (options.id) {
        path += "/" + options.id;
      }
      api.putJson(path, data, {
        success: options.success,
        fail: options.fail
      });
    }

    create(data, success_fnc, fail_fnc) {
      return this.put(data, {
        success: success_fnc,
        fail: fail_fnc
      });
    }

    save(data, success_fnc, fail_fnc) {
      return this.put(data, {
        id: data.id,
        success: success_fnc,
        fail: fail_fnc
      });
    }

    test(type, data, success_fnc, fail_fnc) {
      var path;
      path = this.testPath + "/" + type;
      api.putJson(path, data, {
        success: success_fnc,
        fail: fail_fnc,
        method: "POST"
      });
    }

    del(id, success_fnc, fail_fnc) {
      var path;
      path = this.delPath + "/" + id;
      api.deleteJson(path, {
        success: success_fnc,
        fail: fail_fnc
      });
    }

    permission() {
      if (api.config.admin === true) {
        return api.permissions.ALL;
      }
      return api.permissions.NONE;
    }

    getPermissions(id, success_fnc, fail_fnc) {
      var path;
      path = "permissions/" + this.path + '/' + id;
      return this._requestPath(path, {
        cacheKey: ".",
        success: success_fnc,
        fail: fail_fnc
      });
    }

    addPermission(id, type, itemId, perm, success_fnc, fail_fnc) {
      var data, path;
      path = "permissions/" + this.path + '/' + id + '/' + type + '/add/' + itemId;
      data = {
        perm: perm
      };
      return api.putJson(path, data, {
        success: success_fnc,
        fail: fail_fnc
      });
    }

    revokePermissions(itemIds, success_fnc, fail_fnc) {
      var data, path;
      path = "permissions/revoke";
      data = {
        items: itemIds
      };
      return api.putJson(path, data, {
        success: success_fnc,
        fail: fail_fnc
      });
    }

    types(success_fnc, fail_fnc) {
      return this._requestPath(this.typesPath, {
        cacheKey: this.typesPath,
        success: success_fnc
      });
    }

    gui(typeName, success_fnc, fail_fnc) {
      var path;
      path = null;
      if (typeName != null) {
        path = [this.guiPath, typeName].join("/");
      } else {
        path = this.guiPath;
      }
      return this._requestPath(path, {
        cacheKey: '.',
        success: success_fnc,
        fail: fail_fnc
      });
    }

    tableInfo(success_fnc, fail_fnc) {
      var path;
      success_fnc = success_fnc || function() {
        api.doLog("success not provided for tableInfo");
      };
      path = this.tableInfoPath;
      this._requestPath(path, {
        cacheKey: path,
        success: success_fnc,
        fail: fail_fnc
      });
    }

    detail(id, child, options) {
      options = options || {};
      return new DetailModelRestApi(this, id, child, options);
    }

  };

  DetailModelRestApi = class DetailModelRestApi extends BasicModelRest {
    constructor(parentApi, parentId, model, options) {
      super([parentApi.path, parentId, model].join("/"));
      this.moptions = options;
    }

    permission() {
      if (this.moptions.permission != null) {
        return this.moptions.permission;
      } else {
        return api.permissions.ALL;
      }
    }

    create(data, success_fnc, fail_fnc) {
      return this.put(data, {
        success: success_fnc,
        fail: fail_fnc
      });
    }

    save(data, success_fnc, fail_fnc) {
      return this.put(data, {
        id: data.id,
        success: success_fnc,
        fail: fail_fnc
      });
    }

    types(success_fnc, fail_fnc) {
      if (this.moptions.types) {
        this.moptions.types(success_fnc, fail_fnc);
      } else {
        super.types(success_fnc, fail_fnc);
      }
    }

    // Generic "Invoke" method (with no args, if needed, put them on "method" after "?" as normal url would be
    invoke(method, success_fnc, fail_fnc, options) {
      var meth;
      options = options || {};
      meth = method;
      if (options.params) {
        meth += '?' + options.params;
      }
      return this.get({
        id: meth,
        success: success_fnc,
        fail: fail_fnc
      });
    }

  };

  // Populate api
  api.providers = new BasicModelRest("providers");

  // all services method used in providers
  api.providers.allServices = function(success_fnc, fail_fnc) {
    return this.get({
      id: "allservices",
      success: success_fnc,
      fail: fail_fnc
    });
  };

  api.providers.service = function(id, success_fnc, fail_fnc) {
    return this.get({
      id: "service/" + id,
      success: success_fnc,
      fail: fail_fnc
    });
  };

  api.providers.maintenance = function(id, success_fnc, fail_fnc) {
    return this.get({
      id: id + "/maintenance",
      success: success_fnc,
      fail: fail_fnc
    });
  };

  api.authenticators = new BasicModelRest("authenticators");

  // Search method used in authenticators
  api.authenticators.search = function(id, type, term, success_fnc, fail_fnc) {
    return this.get({
      id: id + "/search?type=" + encodeURIComponent(type) + "&term=" + encodeURIComponent(term),
      success: success_fnc,
      fail: fail_fnc
    });
  };

  api.osmanagers = new BasicModelRest("osmanagers");

  api.transports = new BasicModelRest("transports");

  api.networks = new BasicModelRest("networks");

  api.servicesPools = new BasicModelRest("servicespools");

  api.servicesPools.setFallbackAccess = function(id, fallbackAccess, success_fnc, fail_fnc) {
    return this.get({
      id: id + '/setFallbackAccess?fallbackAccess=' + fallbackAccess,
      success: success_fnc,
      fail: fail_fnc
    });
  };

  api.servicesPools.actionsList = function(id, success_fnc, fail_fnc) {
    return this.get({
      id: id + '/actionsList',
      success: success_fnc,
      fail: fail_fnc
    });
  };

  api.configuration = new BasicModelRest("config");
  api.tconfiguration = new BasicModelRest("tconfig"); //Termidesk config
  api.dashboard = new BasicModelRest("dashboard"); //Termidesk dashboard
  api.monitoring = new BasicModelRest("monitoring"); //Termidesk monitoring
  
  api.gallery = new BasicModelRest("gallery/images");

  api.sPoolGroups = new BasicModelRest("gallery/servicespoolgroups");

  api.system = new BasicModelRest("system");

  api.reports = new BasicModelRest("reports"); // Not fully used, but basic usage is common

  api.calendars = new BasicModelRest("calendars");

  // In fact, reports do not have any type
  api.reports.types = function(success_fnc, fail_fnc) {
    return success_fnc([]);
  };

  api.system.stats = function(type, success_fnc, fail_fnc) {
    return this.get({
      id: "stats/" + type,
      success: success_fnc,
      fail: fail_fnc
    });
  };

  return;

}).call(this);
