(function($) {
"use strict";
	
	var termidesk = this.termidesk;
	termidesk.dashboard = termidesk.dashboard != null ? termidesk.dashboard : {};
	
	const dashboard = termidesk.dashboard
	
	dashboard.renderAggregateCards = function(jContainer, cards) {
		for (const name in cards) {
    		const cardTpl = dashboard.renderCardTpl(cards[name]);
        	jContainer.append(cardTpl);
		}
	}
	
	dashboard.renderUtilizationBars = function(jContainer, bars) {
		for (const name in bars) {
    		const tpl = dashboard.renderUtilizationBarTpl(bars[name]);
    		jContainer.append(tpl);
		}
	}
	
	dashboard.renderCharts = function(jContainer, charts) {
		for (const name in charts) {
    		const item = charts[name];
    		const chartType = item.type;
    		if ( chartType === 'single-metric') {
    			const tpl = dashboard.renderSingleMetricCardTpl(item);
    			jContainer.append(tpl);
	        	
    			const chartElem = jContainer.find(`[data-js=${item.id}] [data-js=chart]`).get(0);
    			
	        	if ( item.chart.type === 'relationships' ) {
	        		dashboard.setupRelationshipsDonutChart(chartElem, item.chart);
	        	}
	        	
	        	if ( item.chart.type === 'utilization' ) {
	        		dashboard.setupUtilizationDonutChart(chartElem, item.chart);
	        	}
			    
    		}
    		
		}
	}
	
	dashboard.renderCardTpl = function({
	    	count,
	    	label,
	    	url,
	    	icon = 'fa-shield',
	    	notifications = [],
	    }) {
		
		const ntpls = []
		for ( const notification of notifications ) {
			if ( notification.href === '' ) {
				const tpl = `
	    		<span class="card-pf-aggregate-status-notification" ><span 
	    		data-toggle="tooltip" title="${notification.label}"><span 
	    		class="${notification.icon}"> ${notification.count}</span></span></span>
	    		`
	    		ntpls.push(tpl);
			} else {
	    		const tpl = `
	    		<span class="card-pf-aggregate-status-notification" ><span data-toggle="tooltip" title="${notification.label}"><a
				href="${notification.href}"><span class="${notification.icon}"> ${notification.count}</span></a></span></span>
	    		`
	    		ntpls.push(tpl);
			}
		}
		
		const notifStr = ntpls.join('');
		
	 	const tpl = `
	    	<div class="col-xs-6 col-sm-3 col-md-3">
		    	<div class="card-pf card-pf-accented card-pf-aggregate-status">
					<h2 class="card-pf-title">
						<span class="${icon}"></span><span
							class="card-pf-aggregate-status-count">${count}</span> <a
							href="${url}">${label}</a>
					</h2>
					<div class="card-pf-body">
						<p class="card-pf-aggregate-status-notifications" style="cursor:default;">${notifStr}
						</p>
					</div>
				</div>
			</div>
		` 
	/*     	const miniTpl = `
	    	<div class="col-xs-6 col-sm-6 col-md-4 col-lg-3">
		    	<div class="card-pf card-pf-accented card-pf-aggregate-status card-pf-aggregate-status-mini">
			  	  <h2 class="card-pf-title">
			  	    <span class="${icon}"></span>
			  	    <span class="card-pf-aggregate-status-count">${count}</span> <a
					href="${url}">${label}</a>
			  	  </h2>
			  	  <div class="card-pf-body">
			  	    <p class="card-pf-aggregate-status-notifications">
			  	      <span class="card-pf-aggregate-status-notification"><a
						href="${url}"><span class="fa fa-eye"></span></a></span>
			  	    </p>
			  	  </div>
			  	</div>
			</div>
		` */
		
		
		return tpl;
	}
	
	const renderUtilizationBarPart = function({
			description,
			usage_label,
			data = {now,max},
		}) {
		const percent = (100 * data.now / data.max).toFixed(1);
		const percent_remaining = (100 - percent).toFixed(1); 
		
		let bar_style = 'progress-bar-success';
		if ( percent > 70) {
			bar_style = 'progress-bar-warning';
		}
		if ( percent > 90) {
			bar_style = 'progress-bar-danger';
		}
		
		const tpl = `
			<div class="progress-container">
				<div class="progress-description">${description}</div>
				<div class="progress progress-label-top-right">
					<div class="progress-bar ${bar_style}" role="progressbar"
						aria-valuenow="${percent}" aria-valuemin="0" aria-valuemax="100"
						style="width: ${percent}%;max-width:100%;" data-toggle="tooltip" title="${percent}%">
						<span><strong>${usage_label}</strong></span>
					</div>
					<div class="progress-bar progress-bar-remaining"
						role="progressbar" aria-valuenow="${percent_remaining}" aria-valuemin="0"
						aria-valuemax="100" style="width: ${percent_remaining}%;" data-toggle="tooltip"
						title="${percent_remaining}% доступно">
						<span class="sr-only">${percent_remaining}%</span>
					</div>
				</div>
			</div>
		`
		
		return tpl;
	}
	
	dashboard.renderUtilizationBarTpl = function({
    	title,
    	bars = []
    }) {
	
	const rendered_bars = [];
	
	for (const bar_context of bars) {
		const rendered_bar = renderUtilizationBarPart(bar_context);
		rendered_bars.push(rendered_bar);
	}
	
	const rendered_bars_str = rendered_bars.join('');
	
	const miniTpl = `
		<div class="col-xs-6 col-sm-6 col-md-6">
			<div class="card-pf">
				<div class="card-pf-heading">
					<h2 class="card-pf-title">${title}</h2>
				</div>
				<div class="card-pf-body">
					${rendered_bars_str}
				</div>
			</div>
		</div>
	`
	
	return miniTpl;
}
	
	dashboard.renderSingleMetricCardTpl = function({
		id,
		title,
		count,
		description = {
			line1,
			line2,
		},
		chart: {
			type,
			data = []
		},
	    }) {
		
		const miniTpl = `
			<div class="col-xs-6 col-sm-6 col-md-6">
				<div class="card-pf card-pf-utilization" data-js="${id}">
				  <h2 class="card-pf-title">${title}</h2>
				  <div class="card-pf-body">
				    <p class="card-pf-utilization-details">
				      <span class="card-pf-utilization-card-details-count">${count}</span>
				      <span class="card-pf-utilization-card-details-description">
				        <span class="card-pf-utilization-card-details-line-1">${description.line1}</span>
				        <span class="card-pf-utilization-card-details-line-2">${description.line2}</span>
				      </span>
				    </p>
				    <div data-js="chart"></div>
				  </div>
				</div>
			</div>
		`
		return miniTpl;
	}

	dashboard.setupRelationshipsDonutChart = function(container, chartDef) {
    	const c3ChartDefaults = $().c3ChartDefaults();
	    const donutConfig = c3ChartDefaults.getDefaultRelationshipDonutConfig();
	    
	    donutConfig.bindto = container;
	    const columns = [];
	    for (const item of chartDef.data) {
	    	columns.push([item.label, item.value])
	    }

	    const donutData = {
	        type: "donut",
	        columns: columns,
	        order: null
	    };
	    
	    donutConfig.data = donutData;
	    donutConfig.legend = {
	    	show: true,
	    	position: 'right'
	    };
		donutConfig.tooltip = {
    	    contents: $().pfDonutTooltipContents
    	};
	    const chart = c3.generate(donutConfig);
	    $().pfSetDonutChartTitle(container, chartDef.count, chartDef.title);
    }
	    
	dashboard.setupUtilizationDonutChart = function(container, chartDef) {
    	const c3ChartDefaults = $().c3ChartDefaults();
	    const donutConfig = c3ChartDefaults.getDefaultDonutConfig('A');
	    
	    donutConfig.bindto = container;

	    const colorPattern = [];
	    const columns = [];

	    for (const item of chartDef.data) {
	    	columns.push([item.label, item.value])
	    	colorPattern.push(item.color);
	    }
	    
	    donutConfig.color =  {
	    	pattern: colorPattern
        };

	    const donutData = {
	        type: "donut",
	        columns: columns,
	        order: null
	    };
	    
	    donutConfig.data = donutData;
	    donutConfig.tooltip = {
            contents: function (d) {
              return '<span class="donut-tooltip-pf" style="white-space: nowrap;">' +
                      Math.round(d[0].ratio * 100) + '%' + ` ${chartDef.units} ` + d[0].name +
                      '</span>';
            }
        };
	    const chart = c3.generate(donutConfig);
	    $().pfSetDonutChartTitle(container, chartDef.count, chartDef.title);
    }
	
}).call(this, jQuery);