(function($) {
"use strict";
	
	var termidesk = this.termidesk;
	termidesk.forms = termidesk.forms != null ? termidesk.forms : {};
	
	const forms = termidesk.forms
	
	forms.validate = function( jForm, notifyError ) {
		let success = true;
		
		
		const images = jForm.find('input[type=file][data-type=imgchoice]');
		images.each( function(index) {
			const imgFile =$(this);
			
			if (! ('img' in imgFile.get(0).dataset) ) {
				notifyError( { responseText: 'Необходимо загрузить изображение' } );
				success = false;
			}
		});
		
		return success;
	}
	
	forms.toJSON = function(jForm) {
		const checkboxes = jForm.find('input[type=checkbox]');
		const multipleSelects = jForm.find('select[multiple]');
		const images = jForm.find('input[type=file][data-type=imgchoice]');
		const passwords = jForm.find('input[type=password]');

		const formJson = forms.objectifyForm( jForm.serializeArray() )
		
		checkboxes.each(function(index) {
			const cb = $(this);
			if ( cb.prop('checked') ) {
				formJson[cb.prop('name')] = true;
			} else {
				formJson[cb.prop('name')] = false;
			}
		});
		
		multipleSelects.each( function(index) {
			const sel = $(this);
			if ( sel.val().length == 0) {
				formJson[this.name]=[];
			}
			
			// Fix so that array is sent to server
			if ( sel.val().length == 1) {
				formJson[this.name]=sel.val();
			}
		});
		
		images.each( function(index) {
			const imgFile =$(this);
			const reader = new FileReader();
			let imgData = imgFile.get(0).dataset.img;
			imgData = imgData.substring(imgData.indexOf("base64,") + 7);
			formJson[imgFile.prop('name')] = imgData;
		});
		
		passwords.each ( function(index) {
		    const field = $(this)
		    if ( field.val().length > 0 ) {
			// Password is present, submitting as is.
			return;
		    }
		    
		    if ( !field.prop('required')) {
			// Password not set and not required
			// Internal users update case
			// Not submitting empty password
			delete formJson[this.name];
		    }
		})
		return formJson
	}
	
	forms.objectifyForm = function(formArray) {
	  var formObj = {};
	  for (const item of formArray) {
		  if ( formObj.hasOwnProperty(item.name) ) {
			  // Handle multiple values for element
			  if ( Array.isArray(formObj[item.name]) ) {
				  formObj[item.name].push(item.value);
			  } else {
				  const arr = [];
				  arr.push(formObj[item.name]);
				  arr.push(item.value);
				  formObj[item.name] = arr;
			  }
		  } else {
			  formObj[item.name] = item.value;
		  }
	  }
	  return formObj;
	}
	
	forms.buildForm = function(container, items, itemApi) {
		const form = container;
		
		for(const item of items) {
			var labelName = item.gui.label;
			var fieldId = item.name+'_field';
			var defValue = item.gui.defvalue
			let tooltip =''
			if ( item.gui.tooltip ) {
				tooltip = item.gui.tooltip.replace(/"/g, '&quot;');
			}
			let readonly = '';
			let disabled = '';
			if ( item.gui.rdonly ) {
				readonly = 'readonly';
				disabled = 'disabled';
			}
			var required = ''
			if (item.gui.required) {
				required = 'required';
			}
			var nameAttr = item.name;
			
			var minAttr = '';
			var minVal = 0;
			if (item.gui.minValue) {
				minAttr = 'data-minval="'+item.gui.minValue+'"';
				minVal = item.gui.minValue;
			}
			
			var maxAttr = '';
			var maxVal = 987654321;
			if (item.gui.maxValue) {
				maxAttr = 'data-maxval="'+item.gui.minValue+'"';
				maxVal = item.gui.maxValue;
			}
			
    		if ( (item.gui.type == 'text') || (item.gui.type == 'password') ) {
    			let searchCls = '';
    			let autocompleteStr = '';
    			const multiline = item.gui.multiline
    			
    			if ( item.gui.search === true ) {
    				searchCls = 'typeahead';
    				autocompleteStr = 'autocomplete="off"'
    			}
    			
    			let tpl = '';
    			
    			if (multiline > 0 ) {
	    			tpl = `
		    			<div class="form-group">
							<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
							<div class="col-sm-9">
								<textarea id="${fieldId}" rows="${multiline}" title="${tooltip}" data-toggle="tooltip" placeholder="${tooltip}" name="${nameAttr}" ${readonly} ${required} ${autocompleteStr} class="form-control ${searchCls}">${defValue}</textarea>
							</div>
						</div>`
    			} else {
    				tpl = `
						<div class="form-group">
							<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
							<div class="col-sm-9">
								<input type="${item.gui.type}" id="${fieldId}" title="${tooltip}" data-toggle="tooltip" placeholder="${tooltip}" name="${nameAttr}" ${readonly} ${required} ${autocompleteStr} value="${defValue}" class="form-control ${searchCls}">
							</div>
						</div>`
    			}
				
    			
				form.append(tpl);
					
				if ( item.gui.search === true ) {
					const jfld = form.find('[name='+nameAttr+']')
					const fld = jfld.get(0);
		    		//console.log(fld.name);
		    		
		    		const search_success = function(response) {
			    		jfld.typeahead({
			  			  source: response,
			  			  autoSelect: true
			  			});
			    	}
		    		
		    		const fail_fn = function(response) {
		    	    	console.log(response);
		    	    }
		    		
		    		itemApi.search(jfld.attr('name'), fld.value, search_success, fail_fn);

				};
				
    		}
    		
    		if (item.gui.type == 'choice') {
    			const values = []
    			for (const val of item.gui.values) {
    				const id = val.id;
    				const text = val.text;
    				let selectedStr = '';
    				if ( defValue === id ) {
    					selectedStr = 'selected'
    				}
    				const option = `<option ${selectedStr} value="${id}">${text}</option>`
    				values.push(option)
    			}
    			
    			const optionsStr = values.join('');
    			
    			var tpl = `
	    			<div class="form-group">
	    				<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
	    				<div class="col-sm-9">
	    					<span title="${tooltip}" data-toggle="tooltip">
			    			<select id="${fieldId}" name="${nameAttr}" ${readonly} ${disabled} ${required} data-js="selectpicker" data-default="${defValue}" class="form-control">
			    			  ${optionsStr}
			    			</select>
			    			</span>
		    			</div>
		    		</div>`
				form.append(tpl);
    		}
    		
    		if (item.gui.type == 'multichoice') {
    			const values = []
    			for (const val of item.gui.values) {
    				const id = val.id;
    				const text = val.text;
    				let selectedStr = '';
    				for(const defval of defValue) {
        				if ( defval === id ) {
        				    selectedStr = 'selected'
        				} else {
        				    if ( typeof defval === 'object' ) {
        					if ( defval.id === id ) {
        	    				    selectedStr = 'selected'
        	    				}
        				    }
        				}
    				}
    				const option = `<option ${selectedStr} value="${id}">${text}</option>`
    				values.push(option)
    			}
    			
    			const optionsStr = values.join('');
    			
    			var tpl = `
	    			<div class="form-group">
	    				<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
	    				<div class="col-sm-9">
	    					<span title="${tooltip}" data-toggle="tooltip">
			    			<select id="${fieldId}" multiple name="${nameAttr}" ${readonly} ${disabled} ${required} data-js="selectpicker" data-default="${defValue}" class="form-control">
			    			  ${optionsStr}
			    			</select>
			    			</span>
		    			</div>
		    		</div>`
				form.append(tpl);
    		}
    		
    		if (item.gui.type == 'numeric') {
    			var tpl = `
    				<div class="form-group">
						<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
						<div class="col-sm-9">
							<span title="${tooltip}" data-toggle="tooltip">
							<input type="numeric" data-js="touchspin" id="${fieldId}" name="${nameAttr}" ${readonly} ${required} value="${defValue}" class="form-control">
							</span>
						</div>
					</div>`
				form.append(tpl);
				form.find('[name='+nameAttr+']').TouchSpin({
					min: -999999999,
					max: 999999999,
				});
    		}
    		
    		if (item.gui.type == 'hidden') {
    			var tpl = `
					<input type="hidden" id="${fieldId}" name="${nameAttr}" ${readonly} ${required} value="${defValue}">`
				form.append(tpl);
    		}
    		
    		if (item.gui.type == 'checkbox') {
    			let checkedStr = '';
    			if ( defValue === true) {
    				checkedStr = 'checked';
    			}
    			
    			let onText = 'Да';
    			let offText = 'Нет';
    			
    			if ( item.gui.hasOwnProperty('on_label') ) {
    				onText = item.gui.on_label;
    			}
    			
    			if ( item.gui.hasOwnProperty('off_label') ) {
    				offText = item.gui.off_label;
    			}
    				
    			var tpl = `
    				<div class="form-group">
						<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
						<div class="col-sm-9">
							<span title="${tooltip}" data-toggle="tooltip">
							<input type="checkbox" id="${fieldId}" name="${nameAttr}" ${readonly} ${required} ${checkedStr} class="form-control">
							</span>
						</div>
					</div>`
				form.append(tpl);
				const cbField = form.find('[name='+nameAttr+']');
				cbField.bootstrapSwitch({
					onText: onText,
					offText: offText
				});
    		}
    		
    		if ( item.gui.type == 'imgchoice') {
    			if ( item.gui.values.length !== 0 ) {
    				const values = []
        			for (const val of item.gui.values) {
        				const id = val.id;
        				const text = val.text;

        				let selectedStr = '';
        				if ( defValue === id ) {
        					selectedStr = 'selected'
        				}
        				const option = `<option ${selectedStr} value="${id}" data-content="<img style='max-width:24px; max-height:24px;' src='data:image/png;base64,${val.img}'/> ${text}">${text}</option>`
        				values.push(option)
        			}
        			
        			const optionsStr = values.join('');
        			
        			var tpl = `
    	    			<div class="form-group">
    	    				<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
    	    				<div class="col-sm-9">
    	    					<span title="${tooltip}" data-toggle="tooltip">
    			    			<select id="${fieldId}" name="${nameAttr}" ${readonly} ${disabled} ${required} data-js="selectpicker" data-default="${defValue}" class="form-control">
    			    			  ${optionsStr}
    			    			</select>
    			    			</span>
    		    			</div>
    		    		</div>`
    				form.append(tpl);
    			} else {
    				const tpl = `
	    				<div class="form-group">
	    					<label class="col-sm-3 control-label" for="${fieldId}">${labelName}</label>
	    					<div class="col-sm-9">
				    			<label class="btn btn-default" for="${fieldId}">
					    		    <span data-js="lbl-text">Выберите изображение</span><input type="file" data-type="imgchoice" style="display:none;" id="${fieldId}" name="${nameAttr}" ${required}>
					    		</label>
					    		<div data-js="img-preview" for="${fieldId}" style="width: 256px;height:256px;">
					    			<img style="max-width: 256px;max-height:256px;"/>
					    		</div>
			    			</div>
			    		</div>`
			    	form.append(tpl);
			    		
			    	const imgField = form.find('[name='+nameAttr+']');
			    	const imgPreview = form.find('[data-js=img-preview][for='+fieldId+']');
			    	const lblText = form.find('label[for='+fieldId+'] [data-js=lbl-text]');
			    	
			    	if ( defValue ) {
			    		imgField.get(0).dataset.img = 'data:image/png;base64,'+defValue;
			    		imgPreview.find('img').get(0).src = imgField.get(0).dataset.img;
			    		lblText.text('Изменить');
			    	}
			    	
			    	const imgChange = function(event) {
			    		const file    = imgField.get(0).files[0];
			    		const reader  = new FileReader();
	
			    		reader.addEventListener("load", function () {
			    		  imgField.get(0).dataset.img = reader.result;
			    		  imgPreview.find('img').get(0).src = reader.result;
			    		}, false);
	
			    		if (file) {
			    		   reader.readAsDataURL(file);
			    		   lblText.text('Изменить');
			    		}
			    	}
			    	
			    	imgField.on('change', imgChange);
    			}
    			
    		}
    		
    		if ( item.gui.hasOwnProperty('fills') ) {
    			const fills = item.gui.fills;
    			const basePath = "gui/callback/" + fills.callbackName;
		      	const parentField = form.find('[name='+nameAttr+']')
				
				parentField.on('change', function(event) {
					const p = [];
					for (const name of fills.parameters) {
						const value = form.find('[name='+name+']').val();
						p.push(name + "=" + encodeURIComponent(value));
					}

					const path = basePath + "?" + p.join("&");
					
					var success_fn = function(response) {
						
						for (const item of response) {
							const field = form.find('[name='+item.name+']');
							const values = [];

							for (const val of item.values) {
			    				const id = val.id;
			    				const text = val.text;
			    				let selectedStr = '';
			    				if ( field.data('default') === id ) {
			    					selectedStr = 'selected'
			    				}
			    				const option = `<option ${selectedStr} value="${id}">${text}</option>`
			    				values.push(option)
			    			}
			    			
			    			const optionsStr = values.join('');
			    			field.empty();
			    			field.append(optionsStr);
			    			field.selectpicker('refresh');
			    			field.trigger("change");
						}
					}
					
					api.getJson(path, {
					  success: success_fn
					});					
				});
				
				parentField.trigger("change");

    		}
    	}
    	
	//$.fn.selectpicker.Constructor.DEFAULTS.noneSelectedText = 'Ничего не выбрано';

		
    	form.find('[data-js=selectpicker]').selectpicker({
    	    noneSelectedText: 'Ничего не выбрано',
    	});
    	
    	form.find('[data-toggle="tooltip"]').tooltip();
	}
	
	forms.populateTable = function(api, jTable, toolbar) {
    
	    const success_fn = function(response) {
	    	const dataSet = []
	    	for (const s of response) {
	  			dataSet.push(s)
			}
	    	const datatable = jTable.DataTable();
	    	datatable.clear();
	        datatable.rows.add(dataSet);
	        datatable.draw();
	        
	        toolbar.disableItemControls();
	    }
	    
	    const fail_fn = function(response) {
	    	console.log(response);
	    }
	    
	    api.overview(success_fn, fail_fn);
    }
	
	forms.getTypeCapabilities = function({
		api,
		type,
		callback,
		}) {
		
		const fail_fn = function(response) {
			console.log(response);
		}
		
		const success_fn = function(items) {
			for (const typedef of items) {
				
				if ( typedef.type == type ) {
					callback(typedef);
				}
			}
		}
		
		api.types(success_fn, fail_fn);
	}
	
	
	forms.populateNewTypes = function(api, jContainer, jModal) {
		
		const modalId = jModal.attr('id');
		const fail_fn = function(response) {
			console.log(response);
		}
		
		const success_fn = function(items) {
			for (const option of items) {
			    	const typename = option.name.replace(/"/g, '&quot;');
				const tpl = `<li><a href="#" data-toggle="modal" data-target="#${modalId}" data-type="${option.type}" data-typename="${typename}">${option.name}</a></li>`
				jContainer.append(tpl);
			}
		}
		
		api.types(success_fn, fail_fn);
	}
	
	forms.requestGui = function(api, itemType, jForm) {
		
		const gui_fn = function(response) {
			
			for ( const field of response ) {
				const fieldName = field.name;
				if ( field.gui.rdonly === true ) {
					// For new item readonly fields are editable
					field.gui.rdonly = false;
				};
			}
			
	    	termidesk.forms.buildForm(jForm, response, api);
	    	
	    }
		
		const fail_fn = function(response) {
	    	console.log(response);
	    }
		
		api.gui(itemType, gui_fn, fail_fn);
	}
	
	forms.requestEditGui = function(itemApi, itemType, itemId, jForm) {
		var fail_fn = function(response) {
	    	console.log(response);
	    }
		
		var gui_fn = function(fields) {
			var setItemValues = function(item) {
				for ( const field of fields ) {
					const fieldName =field.name;
					if ( item.hasOwnProperty(fieldName) ) {
						field.gui.defvalue = item[fieldName];
					};
				}
				termidesk.forms.buildForm(jForm, fields, itemApi);
			}
			
			itemApi.item(itemId, setItemValues, fail_fn);
	    }
		
		itemApi.gui(itemType, gui_fn, fail_fn);
	}
	
	forms.showTestResult = function(jContainer, response, showError) {
		
		if ( response === "ok" ) {
			const successTpl = `
				<div class="alert alert-success alert-dismissable">
				  <button type="button" class="close" data-dismiss="alert" aria-label="Close">
				    <span class="pficon pficon-close"></span>
				  </button>
				  <span class="pficon pficon-ok"></span>
				  Проверка прошла успешно.
				</div>`
			
			jContainer.append(successTpl);
				
		} else {
			showError('Проверка не пройдена. '+ response );
		}
	}
	
	forms.setupTestBtnListener = function(jForm, jMsgContainer, itemApi, itemType) {
		const listener = function(event) {
			event.preventDefault();
			const formJson = termidesk.forms.toJSON( jForm )
			jMsgContainer.empty();
			
			const showErrorFn = forms.setupShowErrorFn(jMsgContainer);
			
			const test_fn = function(response) {
				forms.showTestResult(jMsgContainer, response, showErrorFn);
			}
			 
			itemApi.test(itemType, formJson, test_fn)
		};
		
		return listener;
	};
	
	forms.setupSubmitBtnListener = function(jForm, jMsgContainer) {
		const listener = function (event) {
			event.preventDefault();
			if ( !jForm[0].checkValidity() ) {
				// Trigger browser form validation
				// Create the temporary button, click and remove it
				const tmpSubmit = document.createElement('button');
				jForm[0].appendChild(tmpSubmit)
				tmpSubmit.click();
				jForm[0].removeChild(tmpSubmit)
			} else {
				jMsgContainer.empty();
				jForm.submit();
			};
		};
		
		return listener;
	};
	
	forms.setupShowSuccessFn = function( jMsgContainer ) {
		const fn = function(msg) {
			const failTpl = `
				<div class="alert alert-success alert-dismissable">
				  <button type="button" class="close" data-dismiss="alert" aria-label="Close">
				    <span class="pficon pficon-close"></span>
				  </button>
				  <span class="pficon pficon-ok"></span>
				  ${msg}
				</div>`
			jMsgContainer.append(failTpl);
		};
		return fn;
	}
	
	forms.setupShowErrorFn = function( jMsgContainer ) {
		const fn = function(msg) {
			const failTpl = `
				<div class="alert alert-danger alert-dismissable">
				  <button type="button" class="close" data-dismiss="alert" aria-label="Close">
				    <span class="pficon pficon-close"></span>
				  </button>
				  <span class="pficon pficon-error-circle-o"></span>
				  ${msg}
				</div>`
			jMsgContainer.append(failTpl);
		};
		return fn;
	}
	
	forms.setupFailResponseHandler = function( showError ) {
		const handler = function(response) {
			console.log(response);
			showError(response.responseText);
		};
		return handler;
	};
	
	forms.setupAddModal = function({
		itemApi,
		jTable,
		itemToolbar,
		}) {
		
		const listener = function (event) {
			const button = $(event.relatedTarget) // Button that triggered the modal
			const itemType = button.get(0).dataset.type; // Extract info from data-* attributes
			const itemTypeName = button.get(0).dataset.typename;
			const modal = $(this)
			const form = modal.find('.modal-body form');
			const notifications = modal.find('.modal-body [data-js=notifications]');
			form.empty();
			notifications.empty();
			form.get(0).dataset.type = itemType;
			let modalTitle = 'Добавить новый объект';
			if ( itemTypeName ) {
			    modalTitle = `${modalTitle} (${itemTypeName})`;
			}
			modal.find('.modal-title').text(modalTitle)
			termidesk.forms.requestGui(itemApi, itemType, form);
			
			const withTestBtn = (modal.find('[data-js=test]').length !== 0 );
			if ( withTestBtn ) {
				const testClickListener = termidesk.forms.setupTestBtnListener(form, notifications, itemApi, itemType);
				modal.find('[data-js=test]').off().on( "click", testClickListener );
			}
			
			const submitClickListener = termidesk.forms.setupSubmitBtnListener(form, notifications);
			modal.find('[data-js=submit]').off().on( 'click', submitClickListener );
			
			form.off().on("submit", function( event ) {
				event.preventDefault();
				const form = $(this);
				const showErrorFn = termidesk.forms.setupShowErrorFn(notifications)
				const fail_fn = termidesk.forms.setupFailResponseHandler(showErrorFn);
				
				const validationResult = termidesk.forms.validate( form, showErrorFn);
				
				if (!validationResult) {
					return;
				}
				
				const formJson = termidesk.forms.toJSON( form );

				if ( itemType ) {
					formJson['data_type']=itemType;
				}
				formJson['tags']=[""];
				
				const success_fn = function(response) {
					modal.modal('hide');
					termidesk.forms.populateTable(itemApi, jTable, itemToolbar);
				};
				
				
				itemApi.put(formJson, { success:success_fn, fail: fail_fn })
			});
		};
		
		return listener;
	}
	
	forms.setupEditModal = function({
		itemApi,
		jTable,
		itemToolbar
		}) {
		
		const listener = function (event) {
			const button = $(event.relatedTarget) // Button that triggered the modal
			const itemType = button.get(0).dataset.type;
			const itemId = button.get(0).dataset.id;
			const itemName = button.get(0).dataset.name;
			const modal = $(this)
			const form = modal.find('.modal-body form');
			const notifications = modal.find('.modal-body [data-js=notifications]');
			form.empty();
			notifications.empty();
			let modalTitle = 'Редактировать объект';
			if ( itemName ) {
			    modalTitle = `Редактировать ${itemName}`;
			}
			modal.find('.modal-title').text(modalTitle)
			termidesk.forms.requestEditGui(itemApi, itemType, itemId, form);
			
			const withTestBtn = (modal.find('[data-js=test]').length !== 0 );
			if ( withTestBtn ) {
				const testClickListener = termidesk.forms.setupTestBtnListener(form, notifications, itemApi, itemType);
				modal.find('[data-js=test]').off().on( "click", testClickListener );
			}

			const submitClickListener = termidesk.forms.setupSubmitBtnListener(form, notifications);
			modal.find('[data-js=submit]').off().on( 'click', submitClickListener );
			
			form.off().on("submit", function( event ) {
				event.preventDefault();
				const showErrorFn = termidesk.forms.setupShowErrorFn(notifications)
				const fail_fn = termidesk.forms.setupFailResponseHandler(showErrorFn);
				
				const validationResult = termidesk.forms.validate( form, showErrorFn);
				
				if (!validationResult) {
					return;
				}
				
				const disabledInputs = form.find('[disabled]');
				disabledInputs.each( (index, input) => $(input).prop('disabled', false) )
				
				const formJson = termidesk.forms.toJSON( form );
				
				disabledInputs.each( (index, input) => $(input).prop('disabled', true) )
				
				formJson['id']=itemId;
				if ( itemType ) {
					formJson['data_type']=itemType;
				}
				formJson['tags']=[""];
				
				const success_fn = function(response) {
					modal.modal('hide');
					termidesk.forms.populateTable(itemApi, jTable, itemToolbar);
				};
				
				itemApi.put(formJson, { id: formJson['id'], success:success_fn, fail: fail_fn })
			});
		};
		
		return listener;
	}
	
	forms.setupRemoveModal = function({
		itemApi, 
		jTable, 
		itemToolbar
		}) {
		
		const listener = function (event) {
			const button = $(event.relatedTarget) // Button that triggered the modal

			const itemId = button.get(0).dataset.id;
			const itemName = button.get(0).dataset.name;
			const modal = $(this);

			const notifications = modal.find('.modal-body [data-js=notifications]');
			const msg = modal.find('.modal-body [data-js=message]');
			
			const showErrorFn = termidesk.forms.setupShowErrorFn(notifications)
			
			notifications.empty();
			msg.empty();
			msg.append(`<p>Подтвердите удаление: ${itemName}.</p>`)

			modal.find('.modal-title').text('Удалить объект')
			
			modal.find('[data-js=remove]').off().on( 'click', function( event ) {
				event.preventDefault();
				
				const success_fn = function(response) {
					modal.modal('hide');
					forms.populateTable(itemApi, jTable, itemToolbar);
				};
				
				const fail_fn = forms.setupFailResponseHandler(showErrorFn);
				itemApi.del(itemId, success_fn, fail_fn);
			});
		};
		
		return listener;
	}
	
	forms.setupPublishModal = function({
		itemApi,
		jTable,
		itemToolbar,
		}) {
		
		const listener = function (event) {
			const button = $(event.relatedTarget) // Button that triggered the modal
			const itemType = button.get(0).dataset.type; // Extract info from data-* attributes
			const modal = $(this)
			const form = modal.find('.modal-body form');
			const notifications = modal.find('.modal-body [data-js=notifications]');
			form.empty();
			notifications.empty();
			form.get(0).dataset.type = itemType;
			modal.find('.modal-title').text('Новая публикация')
			termidesk.forms.requestGui(itemApi, itemType, form);
			
			const submitClickListener = termidesk.forms.setupSubmitBtnListener(form, notifications);
			modal.find('[data-js=submit]').off().on( 'click', submitClickListener );
			
			form.off().on("submit", function( event ) {
				event.preventDefault();
				const form = $(this);
				const showErrorFn = termidesk.forms.setupShowErrorFn(notifications)
				const fail_fn = termidesk.forms.setupFailResponseHandler(showErrorFn);
				
				const validationResult = termidesk.forms.validate( form, showErrorFn);
				
				if (!validationResult) {
					return;
				}
				
				const formJson = termidesk.forms.toJSON( form );

				if ( itemType ) {
					formJson['data_type']=itemType;
				}
				formJson['tags']=[""];
				
				const success_fn = function(response) {
					modal.modal('hide');
					termidesk.forms.populateTable(itemApi, jTable, itemToolbar);
				};
				
				
				itemApi.publish(formJson, { success:success_fn, fail: fail_fn })
			});
		};
		
		return listener;
	}
	
	forms.setupCancelModal = function({
		itemApi,
		jTable,
		itemToolbar,
		}) {
		
		const listener = function (event) {
			const button = $(event.relatedTarget) // Button that triggered the modal
			const itemId = button.get(0).dataset.id;
			const modal = $(this)
			const form = modal.find('.modal-body form');
			const notifications = modal.find('.modal-body [data-js=notifications]');
			const showErrorFn = termidesk.forms.setupShowErrorFn(notifications)
			
			form.empty();
			notifications.empty();
			
			modal.find('.modal-title').text('Подтвердите отмену публикации')
			
			modal.find('[data-js=cancel]').off().on("click", function( event ) {
				event.preventDefault();
				
				const success_fn = function(response) {
					modal.modal('hide');
					forms.populateTable(itemApi, jTable, itemToolbar);
				};
				
				const fail_fn = forms.setupFailResponseHandler(showErrorFn);
				itemApi.cancel(itemId, success_fn, fail_fn);
			});
		};
		
		return listener;
	}
	
	forms.setupTableSelectionHandler = function({
		jTable,
		itemToolbar
		}) {
		
		jTable.DataTable().on( 'select', function ( e, dt, type, indexes ) {
		    if ( type === 'row' ) {
		        const item = jTable.find('tr.selected [data-js=item]')
		        const itemId = item.data('id')
		        const itemType = item.data('type')
		        const itemTypeName = item.data('typename')
		        const itemName = item.data('name')
			const itemMaintenance = item.get(0).dataset.maintenance
				
		        itemToolbar.enableItemControls({
		        	id: itemId,
		        	type: itemType,
		        	typeName: itemTypeName,
		        	name: ( itemName || item.text() ),
		        	maintenance: itemMaintenance,
		        });
		    }
		} );
		
		jTable.DataTable().on( 'deselect', function ( e, dt, type, indexes ) {
		    if ( type === 'row' ) {
		    	itemToolbar.disableItemControls();
		    }
		} ); 
	}
	
	forms.setupItemUI = function({
		itemApi, 
		jTable, 
		modals = {}, 
		itemToolbar
		}) {
		
		if ( modals.add ) {
			modals.add.on('show.bs.modal', forms.setupAddModal({itemApi, jTable, itemToolbar}));
		}
		
		if ( modals.edit ) {
			modals.edit.on('show.bs.modal', forms.setupEditModal({itemApi, jTable, itemToolbar}));
		}
		
		if ( modals.remove ) {
			modals.remove.on('show.bs.modal', forms.setupRemoveModal({itemApi, jTable, itemToolbar}));
		}
		
		if ( modals.publish ) {
			modals.publish.on('show.bs.modal', forms.setupPublishModal({itemApi, jTable, itemToolbar}));
		}
		
		if ( modals.cancel ) {
			modals.cancel.on('show.bs.modal', forms.setupCancelModal({itemApi, jTable, itemToolbar}));
		}
		
		forms.setupTableSelectionHandler({jTable, itemToolbar})
	}

}).call(this, jQuery);