# -*- coding: utf-8 -*-

#
# Copyright (c) 2012 Virtual Cable S.L.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#    * Neither the name of Virtual Cable S.L. nor the names of its contributors
#      may be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''
@author: Adolfo Gómez, dkmaster at dkmon dot com

@modified: Dmitry Dvoeglazov, 13.12.2018
'''
from __future__ import unicode_literals

from django.core.management.base import BaseCommand
from uds.core.util.Config import Config, GLOBAL_SECTION, GlobalConfig
import logging

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Updates configuration values. If section is omitted, global section will be used. Omit whitespaces betwen name, =, and value (they must be a single param)"
    
    def add_arguments(self, parser):
        parser.add_argument('param', nargs='+', type=str, help='Syntax: section1.name1=value1 section2.name2=value2 ...')
    
    def handle(self, *args, **options):
        logger.debug("Handling settings")
        GlobalConfig.initialize()
        try:
            for param in options['param']:
                config = param.decode('utf-8')
                logger.debug('Config: {}'.format(config))
                first, value = config.split('=')
                first = first.split('.')
                if len(first) == 2:
                    mod, name = first
                else:
                    mod, name = GLOBAL_SECTION, first[0]
                if Config.update(mod, name, value) is False:  # If not exists, try to store value without any special parameters
                    Config.section(mod).value(name, value).get()
        except Exception:
            logger.exception('Exception processing {}'.format(args))
