# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import datetime
import functools
import os
import subprocess
import sys

def get_version(version=None):
    version = get_complete_version(version)

    # Формируем части номера версии ПО:
    # main = X.Y[.Z]
    # sub = -dev.<git branch>.<utc timestamp>.<id> - для версий разработчиков
    #     | -{a|b|rc}N - для альфа, бета и других релизов

    main = get_main_version(version)

    sub = ''
    if version[3] == 'alpha' and version[4] == 0:
        git_changeset = get_git_changeset()
        if git_changeset:
            main = ''
            sub = 'dev.%s' % git_changeset

    elif version[3] != 'final':
        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'rc'}
        sub = '-{}{}'.format(mapping[version[3]], str(version[4]))

    return main + sub

def get_main_version(version=None):
    """Return main version (X.Y[.Z]) from VERSION."""
    version = get_complete_version(version)
    parts = 2 if version[2] == 0 else 3
    return '.'.join(str(x) for x in version[:parts])


def get_complete_version(version=None):
    """
    Return a tuple of the app version. If version argument is non-empty,
    check for correctness of the tuple provided.
    """
    if version is None:
        version = VERSION
    else:
        assert len(version) == 5
        assert version[3] in ('alpha', 'beta', 'rc', 'final')

    return version

#@functools.lru_cache()
def get_git_changeset():
    """ Возвращает строку <git branch>.<utc timestamp>.<id> для текущего коммита в git.
        Не предполагается вызов данной функции в рабочей среде, т.е. только для среды разработки
    """
    repo_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    git_branch_proc = subprocess.Popen(
        'git rev-parse --abbrev-ref HEAD',
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        shell=True, cwd=repo_dir, universal_newlines=True,
    )
    git_branch = git_branch_proc.communicate()[0].strip()
    
    git_short_sha_proc = subprocess.Popen(
        'git rev-parse --short HEAD',
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        shell=True, cwd=repo_dir, universal_newlines=True,
    )
    git_short_sha = git_short_sha_proc.communicate()[0].strip()
    
    git_log = subprocess.Popen(
        'git log --pretty=format:%ct --quiet -1 HEAD',
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        shell=True, cwd=repo_dir, universal_newlines=True,
    )
    timestamp = git_log.communicate()[0]
    try:
        timestamp = datetime.datetime.utcfromtimestamp(int(timestamp))
    except ValueError:
        return None
    
    sub = "{}.{}.{}".format(git_branch, timestamp.strftime('%Y%m%d%H%M%S'), git_short_sha)
    return sub

VERSION = (1, 2, 1, 'final', 0)
__version__ = '1.2.1'

if __name__ == "__main__":
    print(get_version(VERSION))