# -*- coding: utf-8 -*-
'''
Created on 16 янв. 2019 г.

@author: dvd
'''
from __future__ import unicode_literals

import json
import websockify
from websockify import websocket
import socket
import urllib
import logging
from termidesk.core.util import wsptoken
import argparse


class TermideskProxyRequestHandler(websockify.ProxyRequestHandler):
    def __init__(self, retsock, address, proxy, *args, **kwargs):
        self._proxy = proxy
        websockify.ProxyRequestHandler.__init__(self, retsock, address, proxy,
                                                *args, **kwargs)

    def get_target(self, target_cfg, path):
        """
        Parses the path, extracts a token, and looks for a valid
        target for that token in the configuration file(s). Returns
        target_host and target_port if successful and sets an ssl_target
        flag.
        """
        print path[2:]
        connection_data = self._proxy._tokenDecoder.decode(path[2:])
        print connection_data
        target_host = connection_data['host']
        target_port = connection_data['port']
        self.server.ssl_target = connection_data['ssl_target']
        return (target_host, target_port)
    
    # Копия метода из websockify.ProxyRequestHandler версии 0.8
    # Необходим для применения патча
    def new_websocket_client(self):
        """
        Called after a new WebSocket connection has been established.
        """
        # Checking for a token is done in validate_connection()

        # Connect to the target
        if self.server.wrap_cmd:
            msg = "connecting to command: '%s' (port %s)" % (" ".join(self.server.wrap_cmd), self.server.target_port)
        elif self.server.unix_target:
            msg = "connecting to unix socket: %s" % self.server.unix_target
        else:
            msg = "connecting to: %s:%s" % (
                                    self.server.target_host, self.server.target_port)

        if self.server.ssl_target:
            msg += " (using SSL)"
        self.log_message(msg)

        tsock = websocket.WebSocketServer.socket(self.server.target_host,
                                                 self.server.target_port,
                connect=True, use_ssl=self.server.ssl_target, unix_socket=self.server.unix_target)
        
        # Патч для отключения алгоритма Nagle
        # Источник: 
        # https://github.com/novnc/websockify/commit/f23780eb42ec9a37753556734d1185c1742a3ae7#diff-4c90134602258d00a1c59d8edabcccef
        self.request.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
        if not self.server.wrap_cmd and not self.server.unix_target:
            tsock.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
        
        self.print_traffic(self.traffic_legend)

        # Start proxying
        try:
            self.do_proxy(tsock)
        except:
            if tsock:
                tsock.shutdown(socket.SHUT_RDWR)
                tsock.close()
                if self.verbose:
                    self.log_message("%s:%s: Closed target",
                            self.server.target_host, self.server.target_port)
            raise

class TermideskWebSocketProxy(websockify.WebSocketProxy):
    """"
    Обертка вокруг websockify прокси для работы с Termidesk
    """

    def __init__(self, tokenDecoder, logger = None, *args, **kwargs):
        self._tokenDecoder = tokenDecoder
        if logger:
            self._logger = logger
        else:
            self._logger = logging.getLogger('TermideskWebSocketProxy')
            
        super(TermideskWebSocketProxy, self).__init__(*args, **kwargs)

    def get_logger(self):
        return self._logger

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Termidesk websocket proxy')
    parser.add_argument('--log', default='', help='Log file')
    parser.add_argument('--verbose', default=False, type=bool, help='Verbose output')
    parser.add_argument('--host', default='127.0.0.1', help='Listen ip address')
    parser.add_argument('--port', default=5099, type=int, help='Listen port')
    parser.add_argument('--verification-key', required=True, help='Path to token data verification key')
    
    args = parser.parse_args()
     
    with open(args.verification_key) as f:
        public_key = f.read()
    
    log_level = logging.INFO
    if args.verbose:
        log_level = logging.DEBUG
    if args.log:    
        logger = logging.basicConfig(filename=args.log,level=log_level)
    else:
        logger = logging.basicConfig(level=log_level)
    
    kwargs = {'token_plugin': 'TokenFile'}
    
    TermideskWebSocketProxy(
        listen_host=args.host,
        listen_port=args.port,
        source_is_ipv6=False,
        verbose=args.verbose,
        tokenDecoder=wsptoken.TokenDecoder(public_key),
        logger=logger,
        daemon=False,
        web=None,
        target_host=None,
        target_port=None,
        wrap_mode='exit',
        wrap_cmd=None,
        RequestHandlerClass=TermideskProxyRequestHandler,
        **kwargs
    ).start_server()